/*
 * Copyright 2008 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 */


/*
 * Band.hpp
 *
 *  Created on: 2010/12/10
 *      Author: HARK Development Team
 */

#ifndef BAND_HPP_
#define BAND_HPP_

#include <DimensionPromise/types/Frequency.hpp>

namespace NSHarkBinaural
{

class BandInformation;

/**
 * 離散フーリエ変換で得られたスペクトル上での周波数バンドを指すクラス.
 * サンプリング周波数Fs, データ数Nで離散フーリエ変換すると、
 * N/2+1個の周波数成分からなるスペクトルが得られる。
 * このクラスのインスタンスは、そのスペクトルを配列とみたときの
 * 配列のインデックスを保持する。
 */
class Band
{
public:
    /**
     * どの周波数バンドも指さない、空のインスタンスを生成する.
     */
    Band()
        : index(-1), bandInfo(0)
    { }

    /**
     * 周波数バンドを指すインスタンスを生成する.
     */
    Band(int index, const BandInformation *bandInfo)
        : index(index), bandInfo(bandInfo)
    { }

    /**
     * インスタンスが指すバンドのインデックスを得る.
     * @return 周波数バンドのインデックス
     */
    int Index() const
    {
        return this->index;
    }

    /**
     * インスタンスが指すバンドの周波数を得る.
     * bandInfoがnullの場合や、indexが不正な値の場合、0を返す。
     * @return 周波数バンドの周波数
     */
    NSDimensionPromise::Frequency Freq() const;

    /**
     * 有効な周波数バンドかどうか.
     * @return 有効ならtrue
     */
    bool IsValid() const
    {
        return this->index >= 0;
    }

private:
    /**
     * インスタンスが指すバンドのスペクトルでのインデックス.
     */
    int index;

    /**
     * 周波数バンド情報を扱うクラスへのポインタ.
     */
    const BandInformation *bandInfo;
};

/**
 * Bandクラスのインスタンスが指す周波数バンドの情報を扱うクラス.
 * Bnadクラスは単純なインデックス値を持つ。
 * このクラスは、そのインデックスに対応する周波数は何Hzなのか、
 * などの情報を提供する。
 */
class BandInformation
{
public:
    /**
     * 周波数バンド情報クラスを生成する.
     * @param fs サンプリング周波数
     * @param ndft 離散フーリエ変換のデータ数
     */
    BandInformation(NSDimensionPromise::Frequency fs, unsigned int ndft);

    /**
     * Bandクラスのインスタンスを生成する.
     * 生成されるインスタンスは、このBandInformationクラスのインスタンスへのポインタを持ち、
     * 引数で与えられる周波数バンドのインデックスを持つ。
     * もし、与えられたインデックスが不正なら、空のインスタンスを返す。
     * @param index 周波数バンドのインデックス
     * @return 生成されたインスタンス
     */
    Band CreateBand(int index) const;

    /**
     * Bandクラスのインスタンスを生成する.
     * 生成されるインスタンスは、このBandInformationクラスのインスタンスへのポインタを持ち、
     * 引数で与えられる周波数で表される周波数バンドのインデックスを持つ。
     * もし、与えられた周波数が不正なら、空のインスタンスを返す。
     * @param f 生成する周波数バンドの周波数
     * @return 生成されたインスタンス
     */
    Band CreateBand(NSDimensionPromise::Frequency f) const;

    /**
     * Bandクラスのインスタンスが指す周波数バンドの周波数を得る.
     * band.Index()が不正な値の場合、0を返す。
     * @param band 周波数バンドを指す変数
     * @return 周波数バンドの周波数
     */
    NSDimensionPromise::Frequency FreqOfBand(const Band& band) const;

private:
    /**
     * サンプリング周波数.
     */
    NSDimensionPromise::Frequency fs;

    /**
     * 離散フーリエ変換のデータ数.
     */
    unsigned int ndft;
};

}

#endif /* BAND_HPP_ */
