/*
 * Copyright 2008 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 */


/*
 * InterauralDifference.cpp
 *
 *  Created on: 2010/11/11
 *      Author: HARK Development Team
 */

#include <cstddef>
#include <valarray>

#include <boost/shared_ptr.hpp>

#include <Vector.h>

#include <InterauralDifference.hpp>
#include <Debug.hpp>
#include <Channel.hpp>
#include <Signal.hpp>
#include <LRSignal.hpp>

using namespace std;

namespace NSHarkBinaural
{

//Logger& InterauralIntensityDifference::logger = Logger::GetLogger(Logger::GLOBAL_LOGGER_NAME);
//Logger& InterauralPhaseDifference::logger = Logger::GetLogger(Logger::GLOBAL_LOGGER_NAME);

InterauralIntensityDifference::InterauralIntensityDifference(
    LRSpectraPointer input, Channel::Kind relativeTo)
    : InterauralDifference<Spectrum::ElementType::value_type>(relativeTo),
      logger(Logger::GetLogger(Logger::GLOBAL_LOGGER_NAME))
{
    LogFinest(logger, "constructor");
    if (input.get() != 0 && input->Left().get() != 0 && input->Right().get() != 0)
    {
        // inputがnullではない、かつinputが指す左右の要素もnullではない
        if (IsEqualSignalSize(input))
        {
            // 左右の信号の要素数が等しい
#if 0
            valarray<Spectrum::ElementType>* spec = 0;
            valarray<Spectrum::ElementType>* base = 0; // 計算の基準側
            switch (relativeTo)
            {
            case Channel::R: // Rチャンネルを基準にする
                spec = &input->Left()->Data();
                base = &input->Right()->Data();
                break;
            case Channel::L: // Lチャンネルを基準にする
                spec = &input->Right()->Data();
                base = &input->Left()->Data();
                break;
            default:
                break;
            }

            if (spec != 0 && base != 0)
            {
                LogFinest(logger, "calculating IID");
                Difference().resize(spec->size());
                for (size_t i = 0; i < spec->size(); ++i)
                {
                    Difference()[i] = Decibel( (*spec)[i] ) - Decibel( (*base)[i] );
                }
            }
#endif

            Spectrum* spec = 0;
            Spectrum* base = 0;
            switch (relativeTo)
            {
            case Channel::R: // Rチャンネルを基準にする
                spec = input->Left().get();
                base = input->Right().get();
                break;
            case Channel::L: // Lチャンネルを基準にする
                spec = input->Right().get();
                base = input->Left().get();
                break;
            default:
                break;
            }

            if (spec != 0 && base != 0)
            {
                LogFinest(logger, "calculating IID");
                Difference().resize(spec->Size());
                for (size_t i = 0; i < spec->Size(); ++i)
                {
                    Difference()[i] = Decibel( spec->At(i) ) - Decibel( base->At(i) );
                }
            }
        }
    }
}


FD::RCPtr<FD::Vector<IID::ElementType> > ToFDVector(IID::ptr_t iid)
{
    // FD::Vectorのよくない点を見つけた。
    // FD::Vectorはstd::vectorをpublic継承してしまっている！
    // std::vectorはデストラクタがvirtualではないので、
    //  std::vector<T> *vect = new FD::Vector<T>();
    //  delete vect;
    // ってやるとダメなのだ。
    // 文句言ってもしかたないのでFD::Vectorを使うけどね・・・
    FD::RCPtr<FD::Vector<IID::ElementType> > result(new FD::Vector<IID::ElementType>(iid->Size()));
    for (size_t i = 0; i < iid->Size(); ++i)
    {
        result->at(i) = iid->operator [](i);
    }
    return result;
}


InterauralPhaseDifference::InterauralPhaseDifference(
    LRSpectraPointer input, Channel::Kind relativeTo)
    : InterauralDifference<Spectrum::ElementType::value_type>(relativeTo),
      logger(Logger::GetLogger(Logger::GLOBAL_LOGGER_NAME))
{
    LogFinest(logger, "constructor");
    if (input.get() != 0 && input->Left().get() != 0 && input->Right().get() != 0)
    {
        // inputがnullではない、かつinputが指す左右の要素もnullではない
        if (IsEqualSignalSize(input))
        {
            // 左右の信号の要素数が等しい
#if 0
            valarray<Spectrum::ElementType>* spec = 0;
            valarray<Spectrum::ElementType>* base = 0; // 計算の基準側
            switch (relativeTo)
            {
            case Channel::R: // Rチャンネルを基準にする
                spec = &input->Left()->Data();
                base = &input->Right()->Data();
                break;
            case Channel::L: // Lチャンネルを基準にする
                spec = &input->Right()->Data();
                base = &input->Left()->Data();
                break;
            default:
                break;
            }

            if (spec != 0 && base != 0)
            {
                LogFinest(logger, "calculating IPD");
                Difference().resize(spec->size());
                for (size_t i = 0; i < spec->size(); ++i)
                {
                    Difference()[i] = PhaseDifference( (*spec)[i], (*base)[i] );
                }
            }
#endif
            Spectrum* spec = 0;
            Spectrum* base = 0; // 計算の基準側
            switch (relativeTo)
            {
            case Channel::R: // Rチャンネルを基準にする
                spec = input->Left().get();
                base = input->Right().get();
                break;
            case Channel::L: // Lチャンネルを基準にする
                spec = input->Right().get();
                base = input->Left().get();
                break;
            default:
                break;
            }

            if (spec != 0 && base != 0)
            {
                LogFinest(logger, "calculating IPD");
                Difference().resize(spec->Size());
                for (size_t i = 0; i < spec->Size(); ++i)
                {
                    Difference()[i] = PhaseDifference( spec->At(i), base->At(i) );
                }
            }
        }
    }
}


FD::RCPtr<FD::Vector<IPD::ElementType> > ToFDVector(IPD::ptr_t ipd)
{
    // FD::Vectorのよくない点を見つけた。
    // FD::Vectorはstd::vectorをpublic継承してしまっている！
    // std::vectorはデストラクタがvirtualではないので、
    //  std::vector<T> *vect = new FD::Vector<T>();
    //  delete vect;
    // ってやるとダメなのだ。
    // 文句言ってもしかたないのでFD::Vectorを使うけどね・・・
    FD::RCPtr<FD::Vector<IPD::ElementType> > result(new FD::Vector<IPD::ElementType>(ipd->Size()));
    for (size_t i = 0; i < ipd->Size(); ++i)
    {
        result->at(i) = ipd->operator [](i);
    }
    return result;
}


}
